"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var application_1 = require("tns-core-modules/application");
var utils = require("tns-core-modules/utils/utils");
var standard_common_1 = require("./standard.common");
__export(require("./standard.common"));
function getLocalBroadcastManagerPackage() {
    return useAndroidX() ? global.androidx.localbroadcastmanager.content : global.android.support.v4.content;
}
function useAndroidX() {
    return global.androidx && global.androidx.appcompat;
}
var StripeConfig = (function (_super) {
    __extends(StripeConfig, _super);
    function StripeConfig() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._paymentConfigurationInitiated = false;
        return _this;
    }
    Object.defineProperty(StripeConfig.prototype, "native", {
        get: function () {
            return this.nativeBuilder.build();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeConfig.prototype, "nativeBuilder", {
        get: function () {
            this.initPaymentConfiguration();
            var shippingRequired = this.requiredShippingAddressFields && this.requiredShippingAddressFields.length !== 0;
            var optionalFields = [];
            if (shippingRequired) {
                if (this.requiredShippingAddressFields.indexOf("address") < 0) {
                    optionalFields.unshift(com.stripe.android.view.ShippingInfoWidget.CustomizableShippingField.ADDRESS_LINE_ONE_FIELD);
                    optionalFields.unshift(com.stripe.android.view.ShippingInfoWidget.CustomizableShippingField.ADDRESS_LINE_TWO_FIELD);
                    optionalFields.unshift(com.stripe.android.view.ShippingInfoWidget.CustomizableShippingField.CITY_FIELD);
                    optionalFields.unshift(com.stripe.android.view.ShippingInfoWidget.CustomizableShippingField.STATE_FIELD);
                    optionalFields.unshift(com.stripe.android.view.ShippingInfoWidget.CustomizableShippingField.POSTAL_CODE_FIELD);
                }
                if (this.requiredShippingAddressFields.indexOf("phone") < 0) {
                    optionalFields.unshift(com.stripe.android.view.ShippingInfoWidget.CustomizableShippingField.PHONE_FIELD);
                }
            }
            return new com.stripe.android.PaymentSessionConfig.Builder()
                .setShippingInfoRequired(shippingRequired)
                .setShippingMethodsRequired(shippingRequired)
                .setOptionalShippingInfoFields(optionalFields);
        },
        enumerable: true,
        configurable: true
    });
    StripeConfig.prototype.initPaymentConfiguration = function () {
        if (!this.publishableKey)
            throw new Error("publishableKey must be set");
        if (this._paymentConfigurationInitiated)
            return;
        com.stripe.android.PaymentConfiguration.init(utils.ad.getApplicationContext(), this.publishableKey);
        this._paymentConfigurationInitiated = true;
    };
    StripeConfig.shared = function () {
        if (!standard_common_1.StripeConfigCommon.instance)
            standard_common_1.StripeConfigCommon.instance = new StripeConfig();
        return standard_common_1.StripeConfigCommon.instance;
    };
    return StripeConfig;
}(standard_common_1.StripeConfigCommon));
exports.StripeConfig = StripeConfig;
var StripeCustomerSession = (function () {
    function StripeCustomerSession() {
        StripeConfig.shared().initPaymentConfiguration();
        com.stripe.android.CustomerSession.initCustomerSession(this.context, createKeyProvider());
        this.native = com.stripe.android.CustomerSession.getInstance();
    }
    Object.defineProperty(StripeCustomerSession.prototype, "context", {
        get: function () {
            return application_1.android.context;
        },
        enumerable: true,
        configurable: true
    });
    return StripeCustomerSession;
}());
exports.StripeCustomerSession = StripeCustomerSession;
function createKeyProvider() {
    return new com.stripe.android.EphemeralKeyProvider({
        createEphemeralKey: function (apiVersion, keyUpdateListener) {
            StripeConfig.shared().backendAPI.createCustomerKey(apiVersion)
                .then(function (key) {
                keyUpdateListener.onKeyUpdate(JSON.stringify(key));
            }).catch(function (e) {
                keyUpdateListener.onKeyUpdateFailure(500, JSON.stringify(e));
            });
        }
    });
}
var StripePaymentSession = (function () {
    function StripePaymentSession(_page, customerSession, amount, currency, listener, prefilledAddress) {
        this.customerSession = customerSession;
        this.currency = currency;
        this.listener = listener;
        var builder = StripeConfig.shared().nativeBuilder;
        if (prefilledAddress) {
            var info = new com.stripe.android.model.ShippingInformation(new com.stripe.android.model.Address.Builder()
                .setLine1(prefilledAddress.line1)
                .setLine2(prefilledAddress.line2)
                .setCity(prefilledAddress.city)
                .setState(prefilledAddress.state)
                .setCountry(prefilledAddress.country)
                .setPostalCode(prefilledAddress.postalCode)
                .build(), prefilledAddress.name, prefilledAddress.phone);
            builder.setPrepopulatedShippingInfo(info);
        }
        var config = builder.build();
        this.native = new com.stripe.android.PaymentSession(this.patchActivity());
        if (!this.native.init(createPaymentSessionListener(this, listener), config)) {
            throw new Error("CustomerSession not initialized");
        }
        this.native.setCartTotal(amount);
        this.receiver = createShippingBroadcastReceiver(this, listener);
        getLocalBroadcastManagerPackage().LocalBroadcastManager.getInstance(application_1.android.foregroundActivity).registerReceiver(this.receiver, new android.content.IntentFilter(com.stripe.android.view.PaymentFlowExtras.EVENT_SHIPPING_INFO_SUBMITTED));
    }
    Object.defineProperty(StripePaymentSession.prototype, "amount", {
        get: function () {
            var data = this.native.getPaymentSessionData();
            return data.getCartTotal() + (data.getShippingMethod() ? data.getShippingMethod().getAmount() : 0);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentSession.prototype, "isPaymentReady", {
        get: function () {
            return this.native.getPaymentSessionData().isPaymentReadyToCharge();
        },
        enumerable: true,
        configurable: true
    });
    StripePaymentSession.prototype.requestPayment = function () {
        var _this = this;
        this.paymentInProgress = true;
        var data = this.native.getPaymentSessionData();
        var shippingMethod = data.getShippingMethod();
        var shippingCost = shippingMethod ? shippingMethod.getAmount() : 0;
        StripeConfig.shared().backendAPI.capturePayment(data.getPaymentMethod().component1(), data.getCartTotal() + shippingCost, createShippingMethod(shippingMethod), createAddress(data.getShippingInformation()))
            .then(function () {
            _this.paymentInProgress = false;
            _this.listener.onPaymentSuccess();
            _this.native.onCompleted();
        }).catch(function (e) {
            _this.listener.onError(100, e);
            _this.paymentInProgress = false;
        });
    };
    StripePaymentSession.prototype.presentPaymentMethods = function () {
        this.native.presentPaymentMethodSelection();
    };
    StripePaymentSession.prototype.presentShipping = function () {
        this.native.presentShippingFlow();
    };
    StripePaymentSession.prototype.patchActivity = function () {
        var activity = application_1.android.foregroundActivity;
        var session = this;
        var oldResultCallback = activity.onActivityResult;
        console.log("oldResultCallback: " + JSON.stringify(oldResultCallback));
        activity.onActivityResult = function (requestCode, resultCode, data) {
            session.native.handlePaymentData(requestCode, resultCode, data);
        };
        var oldDestroyCallback = activity.onDestroy;
        console.log("oldDestroyCallback: " + JSON.stringify(oldDestroyCallback));
        activity.onDestroy = function () {
            session.native.onDestroy();
            getLocalBroadcastManagerPackage().LocalBroadcastManager.getInstance(activity).unregisterReceiver(this.receiver);
        };
        return activity;
    };
    return StripePaymentSession;
}());
exports.StripePaymentSession = StripePaymentSession;
function createPaymentSessionListener(parent, listener) {
    return new com.stripe.android.PaymentSession.PaymentSessionListener({
        onPaymentSessionDataChanged: function (sessionData) {
            if (parent.paymentInProgress)
                return;
            parent.customerSession.native.retrieveCurrentCustomer(new com.stripe.android.CustomerSession.CustomerRetrievalListener({
                onCustomerRetrieved: function (_customer) {
                    parent.selectedPaymentMethod = createPaymentMethod(sessionData.getPaymentMethod());
                    parent.selectedShippingMethod = createShippingMethod(sessionData.getShippingMethod());
                    parent.shippingAddress = createAddress(sessionData.getShippingInformation());
                    var paymentData = {
                        isReadyToCharge: sessionData.isPaymentReadyToCharge(),
                        paymentMethod: parent.selectedPaymentMethod,
                        shippingInfo: parent.selectedShippingMethod,
                        shippingAddress: parent.shippingAddress
                    };
                    listener.onPaymentDataChanged(paymentData);
                },
                onError: function (errorCode, errorMessage) {
                    listener.onError(errorCode, errorMessage);
                }
            }));
        },
        onCommunicatingStateChanged: function (isCommunicating) {
            parent.loading = isCommunicating;
            listener.onCommunicatingStateChanged(isCommunicating);
        },
        onError: function (code, message) {
            listener.onError(code, message);
        }
    });
}
function createShippingBroadcastReceiver(parent, listener) {
    var InternalReceiver = (function (_super) {
        __extends(InternalReceiver, _super);
        function InternalReceiver(parent, listener) {
            var _this = _super.call(this) || this;
            _this.parent = parent;
            _this.listener = listener;
            return global.__native(_this);
        }
        InternalReceiver.prototype.onReceive = function (context, intent) {
            var _this = this;
            var shippingInformation = intent.getParcelableExtra(com.stripe.android.view.PaymentFlowExtras.EXTRA_SHIPPING_INFO_DATA);
            var shippingMethods = this.listener.provideShippingMethods(createAddress(shippingInformation));
            var shippingInfoProcessedIntent = new android.content.Intent(com.stripe.android.view.PaymentFlowExtras.EVENT_SHIPPING_INFO_PROCESSED);
            if (!shippingMethods.isValid) {
                shippingInfoProcessedIntent.putExtra(com.stripe.android.view.PaymentFlowExtras.EXTRA_IS_SHIPPING_INFO_VALID, false);
                shippingInfoProcessedIntent.putExtra(com.stripe.android.view.PaymentFlowExtras.EXTRA_SHIPPING_INFO_ERROR, shippingMethods.validationError);
            }
            else {
                shippingInfoProcessedIntent.putExtra(com.stripe.android.view.PaymentFlowExtras.EXTRA_IS_SHIPPING_INFO_VALID, true);
                var methods_1 = new java.util.ArrayList();
                shippingMethods.shippingMethods.forEach(function (m) { return methods_1.add(createAdShippingMethod(m, _this.parent.currency)); });
                shippingInfoProcessedIntent.putParcelableArrayListExtra(com.stripe.android.view.PaymentFlowExtras.EXTRA_VALID_SHIPPING_METHODS, methods_1);
                shippingInfoProcessedIntent.putExtra(com.stripe.android.view.PaymentFlowExtras.EXTRA_DEFAULT_SHIPPING_METHOD, createAdShippingMethod(shippingMethods.selectedShippingMethod, this.parent.currency));
            }
            getLocalBroadcastManagerPackage().LocalBroadcastManager.getInstance(context).sendBroadcast(shippingInfoProcessedIntent);
        };
        return InternalReceiver;
    }(android.content.BroadcastReceiver));
    return new InternalReceiver(parent, listener);
}
function createPaymentMethod(paymentMethod) {
    if (!paymentMethod)
        return undefined;
    var pmCard = paymentMethod.component8();
    var pmId = paymentMethod.component1();
    if (pmCard)
        return createPaymentMethodFromCard(pmCard, pmId);
    return { label: "Error (103)", image: undefined, templateImage: undefined };
}
function createPaymentMethodFromCard(card, stripeID) {
    var brand = card.component1();
    var last4 = card.component7();
    return {
        label: toCardBrand(brand) + " ..." + last4,
        image: getBitmapFromResource(com.stripe.android.model.Card.getBrandIcon(fixupCardBrand(brand))),
        templateImage: undefined,
        type: "Card",
        stripeID: stripeID,
        brand: toCardBrand(brand)
    };
}
function getBitmapFromResource(resID) {
    var image = application_1.android.foregroundActivity.getResources().getDrawable(resID, null);
    if (image instanceof android.graphics.Bitmap) {
        return image;
    }
    var bitmap = android.graphics.Bitmap.createBitmap(image.getIntrinsicWidth(), image.getIntrinsicHeight(), android.graphics.Bitmap.Config.ARGB_8888);
    var canvas = new android.graphics.Canvas(bitmap);
    image.setBounds(0, 0, canvas.getWidth(), canvas.getHeight());
    image.draw(canvas);
    return bitmap;
}
function createShippingMethod(shipping) {
    if (!shipping)
        return undefined;
    return {
        amount: shipping.getAmount(),
        detail: shipping.getDetail(),
        label: shipping.getLabel(),
        identifier: shipping.getIdentifier()
    };
}
function createAddress(info) {
    if (!info)
        return undefined;
    var address = info.getAddress();
    if (!address)
        return undefined;
    return {
        name: info.getName(),
        line1: address.getLine1(),
        line2: address.getLine2(),
        city: address.getCity(),
        state: address.getState(),
        postalCode: address.getPostalCode(),
        country: address.getCountry(),
        phone: info.getPhone(),
        email: undefined
    };
}
function createAdShippingMethod(method, currency) {
    return new com.stripe.android.model.ShippingMethod(method.label, method.identifier, method.amount, currency, method.detail);
}
function toCardBrand(brand) {
    if (brand)
        switch (brand.toLowerCase()) {
            case 'visa':
                return 'Visa';
            case 'amex':
            case 'americanexpress':
            case 'american express':
                return 'Amex';
            case 'mastercard':
                return 'MasterCard';
            case 'discover':
                return 'Discover';
            case 'jcb':
                return 'JCB';
            case 'diners':
            case 'dinersclub':
            case 'diners club':
                return 'DinersClub';
            case 'unionpay':
            case 'union pay':
                return 'UnionPay';
        }
    return 'Unknown';
}
function fixupCardBrand(brand) {
    if (brand)
        switch (brand.toLowerCase()) {
            case 'visa':
                return 'Visa';
            case 'amex':
            case 'americanexpress':
            case 'american_express':
            case 'american express':
                return 'American Express';
            case 'mastercard':
                return 'MasterCard';
            case 'discover':
                return 'Discover';
            case 'jcb':
                return 'JCB';
            case 'diners':
            case 'dinersclub':
            case 'diners_club':
            case 'diners club':
                return 'Diners Club';
            case 'unionpay':
            case 'union pay':
                return 'UnionPay';
        }
    return 'Unknown';
}
