"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var standard_common_1 = require("./standard.common");
__export(require("./standard.common"));
var StripeConfig = (function (_super) {
    __extends(StripeConfig, _super);
    function StripeConfig() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(StripeConfig.prototype, "native", {
        get: function () {
            if (!this.publishableKey)
                throw new Error("publishableKey must be set");
            var config = STPPaymentConfiguration.sharedConfiguration();
            config.publishableKey = this.publishableKey;
            config.appleMerchantIdentifier = this.appleMerchantID;
            config.requiredBillingAddressFields = this.requiredBillingAddressFields;
            if (this.requiredShippingAddressFields && this.requiredShippingAddressFields.length > 0) {
                var fields_1 = new NSMutableSet({ capacity: 4 });
                this.requiredShippingAddressFields.forEach(function (f) {
                    switch (f) {
                        case "name":
                            fields_1.addObject(STPContactFieldName);
                            break;
                        case "address":
                            fields_1.addObject(STPContactFieldPostalAddress);
                            break;
                        case "phone":
                            fields_1.addObject(STPContactFieldPhoneNumber);
                            break;
                        case "email":
                            fields_1.addObject(STPContactFieldEmailAddress);
                            break;
                    }
                });
                config.requiredShippingAddressFields = fields_1;
            }
            else {
                config.requiredShippingAddressFields = undefined;
            }
            config.companyName = this.companyName;
            return config;
        },
        enumerable: true,
        configurable: true
    });
    StripeConfig.shared = function () {
        if (!standard_common_1.StripeConfigCommon.instance)
            standard_common_1.StripeConfigCommon.instance = new StripeConfig();
        return standard_common_1.StripeConfigCommon.instance;
    };
    return StripeConfig;
}(standard_common_1.StripeConfigCommon));
exports.StripeConfig = StripeConfig;
var StripeCustomerSession = (function () {
    function StripeCustomerSession() {
        this.native = STPCustomerContext.alloc().initWithKeyProvider(StripeKeyProvider.new());
    }
    return StripeCustomerSession;
}());
exports.StripeCustomerSession = StripeCustomerSession;
var StripeKeyProvider = (function (_super) {
    __extends(StripeKeyProvider, _super);
    function StripeKeyProvider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StripeKeyProvider.new = function () {
        return _super.new.call(this);
    };
    StripeKeyProvider.prototype.createCustomerKeyWithAPIVersionCompletion = function (apiVersion, completion) {
        StripeConfig.shared().backendAPI.createCustomerKey(apiVersion)
            .then(function (key) {
            completion(key, null);
        }).catch(function (e) {
            completion(null, createError("CustomerKey", 500, e));
        });
    };
    StripeKeyProvider.ObjCProtocols = [STPCustomerEphemeralKeyProvider];
    return StripeKeyProvider;
}(NSObject));
var StripePaymentSession = (function () {
    function StripePaymentSession(page, customerSession, amount, currency, listener, prefilledAddress) {
        this.page = page;
        this.native = STPPaymentContext.alloc()
            .initWithCustomerContextConfigurationTheme(customerSession.native, StripeConfig.shared().native, STPTheme.defaultTheme());
        this.native.prefilledInformation = STPUserInformation.alloc().init();
        if (prefilledAddress) {
            var addr = STPAddress.alloc().init();
            addr.name = prefilledAddress.name;
            addr.line1 = prefilledAddress.line1;
            addr.line2 = prefilledAddress.line2;
            addr.city = prefilledAddress.city;
            addr.state = prefilledAddress.state;
            addr.country = prefilledAddress.country;
            addr.postalCode = prefilledAddress.postalCode;
            addr.phone = prefilledAddress.phone;
            addr.email = prefilledAddress.email;
            this.native.prefilledInformation.shippingAddress = addr;
        }
        this.native.paymentAmount = amount;
        this.native.paymentCurrency = currency;
        if (listener) {
            this.delegate = StripePaymentDelegate.create(this, listener);
            this.native.delegate = this.delegate;
        }
    }
    Object.defineProperty(StripePaymentSession.prototype, "loading", {
        get: function () {
            return this.native.loading;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentSession.prototype, "isPaymentReady", {
        get: function () {
            return this.native.selectedPaymentOption != null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentSession.prototype, "paymentInProgress", {
        get: function () {
            return this._paymentInProgress;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentSession.prototype, "amount", {
        get: function () {
            return this.native.paymentAmount;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentSession.prototype, "selectedPaymentMethod", {
        get: function () {
            return createPaymentMethod(this.native);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentSession.prototype, "selectedShippingMethod", {
        get: function () {
            return createShippingMethod(this.native);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentSession.prototype, "shippingAddress", {
        get: function () {
            return createAddress(this.native.shippingAddress);
        },
        enumerable: true,
        configurable: true
    });
    StripePaymentSession.prototype.ensureHostViewController = function () {
        if (!this.native.hostViewController)
            this.native.hostViewController = this.page.ios;
    };
    StripePaymentSession.prototype.requestPayment = function () {
        this.ensureHostViewController();
        this._paymentInProgress = true;
        this.native.requestPayment();
    };
    StripePaymentSession.prototype.presentPaymentMethods = function () {
        this.ensureHostViewController();
        this.native.presentPaymentOptionsViewController();
    };
    StripePaymentSession.prototype.presentShipping = function () {
        this.ensureHostViewController();
        this.native.presentShippingViewController();
    };
    return StripePaymentSession;
}());
exports.StripePaymentSession = StripePaymentSession;
var StripePaymentDelegate = (function (_super) {
    __extends(StripePaymentDelegate, _super);
    function StripePaymentDelegate() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StripePaymentDelegate.create = function (parent, listener) {
        var self = _super.new.call(this);
        self.parent = parent;
        self.listener = listener;
        return self;
    };
    StripePaymentDelegate.prototype.paymentContextDidChange = function (paymentContext) {
        var data = {
            isReadyToCharge: paymentContext.selectedPaymentOption != null,
            paymentMethod: createPaymentMethod(paymentContext),
            shippingInfo: createShippingMethod(paymentContext),
            shippingAddress: createAddress(paymentContext.shippingAddress)
        };
        this.listener.onPaymentDataChanged(data);
    };
    StripePaymentDelegate.prototype.paymentContextDidCreatePaymentResultCompletion = function (paymentContext, paymentResult, completion) {
        StripeConfig.shared().backendAPI.capturePayment(paymentResult.paymentMethod.stripeId, paymentContext.paymentAmount, createShippingMethod(paymentContext), createAddress(paymentContext.shippingAddress))
            .then(function () {
            completion(0, null);
        }).catch(function (e) {
            completion(null, createError("PaymentError", 100, e));
        });
    };
    StripePaymentDelegate.prototype.paymentContextDidFailToLoadWithError = function (paymentContext, error) {
        this.listener.onError(error.code, error.localizedDescription);
    };
    StripePaymentDelegate.prototype.paymentContextDidFinishWithStatusError = function (paymentContext, status, error) {
        this.parent._paymentInProgress = false;
        switch (status) {
            case 1:
                if (this.listener.onError)
                    this.listener.onError(error.code, error.localizedDescription);
                break;
            case 0:
                if (this.listener.onPaymentSuccess)
                    this.listener.onPaymentSuccess();
                break;
            case 2:
                if (this.listener.onUserCancelled)
                    this.listener.onUserCancelled();
                break;
        }
    };
    StripePaymentDelegate.prototype.paymentContextDidUpdateShippingAddressCompletion = function (_paymentContext, address, completion) {
        var methods = this.listener.provideShippingMethods(createAddress(address));
        if (!methods) {
            completion(1, createError("ShippingError", 120, "No shipping methods"), null, null);
        }
        else if (!methods.isValid) {
            completion(1, createError("ShippingError", 123, methods.validationError), null, null);
        }
        else {
            var sh_1 = NSMutableArray.alloc().init();
            methods.shippingMethods.forEach(function (m) { return sh_1.addObject(createPKShippingMethod(m)); });
            completion(0, null, sh_1, createPKShippingMethod(methods.selectedShippingMethod));
        }
    };
    StripePaymentDelegate.ObjCProtocols = [STPPaymentContextDelegate];
    return StripePaymentDelegate;
}(NSObject));
function createError(domain, code, error) {
    var userInfo = NSMutableDictionary.alloc().init();
    userInfo.setValueForKey(error, NSLocalizedDescriptionKey);
    return new NSError({
        domain: domain, code: code, userInfo: userInfo
    });
}
function createPaymentMethod(paymentContext) {
    if (!paymentContext.selectedPaymentOption)
        return undefined;
    var pmt = paymentContext.selectedPaymentOption;
    var type;
    var stripeId;
    var brand;
    if (pmt.isKindOfClass(STPApplePayPaymentOption)) {
        type = "ApplePay";
        stripeId = undefined;
        brand = undefined;
    }
    else if (pmt.isKindOfClass(STPPaymentMethod)) {
        var pm = pmt;
        if (pm.type === 0) {
            type = "Card";
            stripeId = pm.stripeId;
            brand = STPCard.stringFromBrand(pm.card.brand);
        }
    }
    return {
        label: pmt.label,
        image: pmt.image,
        templateImage: pmt.templateImage,
        type: type,
        stripeID: stripeId,
        brand: brand
    };
}
function createShippingMethod(paymentContext) {
    if (!paymentContext.selectedShippingMethod)
        return undefined;
    return {
        amount: paymentContext.selectedShippingMethod.amount.doubleValue * 100,
        detail: paymentContext.selectedShippingMethod.detail,
        label: paymentContext.selectedShippingMethod.label,
        identifier: paymentContext.selectedShippingMethod.identifier
    };
}
function createPKShippingMethod(method) {
    var m = PKShippingMethod.alloc().init();
    m.amount = NSDecimalNumber.alloc().initWithDouble(method.amount / 100);
    m.detail = method.detail;
    m.label = method.label;
    m.identifier = method.identifier;
    return m;
}
function createAddress(address) {
    if (!address)
        return undefined;
    return {
        name: address.name,
        line1: address.line1,
        line2: address.line2,
        city: address.city,
        state: address.state,
        postalCode: address.postalCode,
        country: address.country,
        phone: address.phone,
        email: address.email
    };
}
