"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var application_1 = require("tns-core-modules/application");
var utils = require("tns-core-modules/utils/utils");
var stripe_common_1 = require("./stripe.common");
__export(require("./stripe.common"));
var Stripe = (function () {
    function Stripe(apiKey, stripeAccountId) {
        this._apiKey = apiKey;
        this._stripeAccountId = stripeAccountId;
    }
    Stripe.prototype.setStripeAccount = function (accountId) {
        this._stripeAccountId = accountId;
    };
    Object.defineProperty(Stripe.prototype, "stripe", {
        get: function () {
            if (!this._stripe) {
                this._stripe = new com.stripe.android.Stripe(utils.ad.getApplicationContext(), this._apiKey, this._stripeAccountId);
            }
            return this._stripe;
        },
        enumerable: true,
        configurable: true
    });
    Stripe.prototype.createToken = function (card, cb) {
        if (!card) {
            if (typeof cb === 'function') {
                cb(new Error('Invalid card'), null);
            }
            return;
        }
        this.stripe.createToken(card.native, new com.stripe.android.ApiResultCallback({
            onSuccess: function (token) {
                if (typeof cb === 'function') {
                    var newToken = {
                        id: token.getId(),
                        bankAccount: token.getBankAccount(),
                        card: Card.fromNative(token.getCard()),
                        created: new Date(token.getCreated().toString()),
                        livemode: token.getLivemode(),
                        android: token,
                        ios: null
                    };
                    cb(null, newToken);
                }
            },
            onError: function (error) {
                if (typeof cb === 'function') {
                    cb(new Error(error.getLocalizedMessage()), null);
                }
            }
        }));
    };
    Stripe.prototype.createSource = function (card, cb) {
        if (!card) {
            if (typeof cb === 'function') {
                cb(new Error('Invalid card'), null);
            }
            return;
        }
        var cardSourceParams = com.stripe.android.model.SourceParams.createCardParams(card.native);
        try {
            this.stripe.createSource(cardSourceParams, new com.stripe.android.ApiResultCallback({
                onSuccess: function (source) {
                    if (typeof cb === 'function') {
                        var newSource = {
                            id: source.getId(),
                            amount: source.getAmount().longValue(),
                            card: card,
                            clientSecret: source.getClientSecret(),
                            created: new Date(source.getCreated().toString()),
                            currency: source.getCurrency(),
                            livemode: source.isLiveMode().booleanValue(),
                            metadata: source.getMetaData()
                        };
                        cb(null, newSource);
                    }
                },
                onError: function (error) {
                    cb(new Error(error.getLocalizedMessage()), null);
                }
            }));
        }
        catch (error) {
            if (typeof cb === 'function') {
                cb(new Error(error.localizedDescription), null);
            }
        }
    };
    Stripe.prototype.createPaymentMethod = function (card, cb) {
        if (!card) {
            if (typeof cb === 'function') {
                cb(new Error('Invalid card'), null);
            }
            return;
        }
        var cardParams = new com.stripe.android.model.PaymentMethodCreateParams.Card.Builder();
        if (card.cvc)
            cardParams.setCvc(card.cvc);
        if (card.expMonth)
            cardParams.setExpiryMonth(new java.lang.Integer(card.expMonth));
        if (card.expYear)
            cardParams.setExpiryYear(new java.lang.Integer(card.expYear));
        if (card.number)
            cardParams.setNumber(card.number);
        var billing = new com.stripe.android.model.PaymentMethod.BillingDetails.Builder();
        var addr = new com.stripe.android.model.Address.Builder();
        if (card.addressLine1)
            addr.setLine1(card.addressLine1);
        if (card.addressLine2)
            addr.setLine2(card.addressLine2);
        if (card.addressCity)
            addr.setCity(card.addressCity);
        if (card.addressState)
            addr.setState(card.addressState);
        if (card.addressZip)
            addr.setPostalCode(card.addressZip);
        if (card.addressCountry)
            addr.setCountry(card.addressCountry);
        billing.setAddress(addr.build());
        var params = com.stripe.android.model.PaymentMethodCreateParams.create(cardParams.build(), billing.build());
        try {
            var apiResultCallback = new com.stripe.android.ApiResultCallback({
                onSuccess: function (result) {
                    cb(null, PaymentMethod.fromNative(result));
                },
                onError: function (error) {
                    cb(new Error(error.localizedDescription), null);
                }
            });
            this.stripe.createPaymentMethod(params, apiResultCallback);
        }
        catch (error) {
            if (typeof cb === 'function') {
                cb(new Error(error.localizedDescription), null);
            }
        }
    };
    Stripe.prototype.retrievePaymentIntent = function (clientSecret, cb) {
        try {
            var pi = this.stripe.retrievePaymentIntentSynchronous(clientSecret);
            cb(null, StripePaymentIntent.fromNative(pi));
        }
        catch (error) {
            cb(new Error(error.localizedDescription), null);
        }
    };
    Stripe.prototype.confirmSetupIntent = function (paymentMethodId, clientSecret, cb) {
        var _this = this;
        try {
            var activity = application_1.android.foregroundActivity;
            var resultCb_1 = new com.stripe.android.ApiResultCallback({
                onSuccess: function (result) {
                    cb(null, StripeSetupIntent.fromNative(result.getIntent()));
                },
                onError: function (error) {
                    cb(new Error(error.localizedDescription), null);
                }
            });
            activity.onActivityResult = function (requestCode, resultCode, data) {
                _this.stripe.onSetupResult(requestCode, data, resultCb_1);
            };
            this.stripe.confirmSetupIntent(activity, new StripeSetupIntentParams(paymentMethodId, clientSecret).native);
        }
        catch (error) {
            cb(new Error(error.localizedDescription), null);
        }
    };
    Stripe.prototype.authenticateSetupIntent = function (clientSecret, returnUrl, cb) {
        var _this = this;
        var activity = application_1.android.foregroundActivity;
        var resultCb = new com.stripe.android.ApiResultCallback({
            onSuccess: function (result) {
                cb(null, StripeSetupIntent.fromNative(result.getIntent()));
            },
            onError: function (error) {
                cb(new Error(error.localizedDescription), null);
            }
        });
        activity.onActivityResult = function (requestCode, resultCode, data) {
            _this.stripe.onSetupResult(requestCode, data, resultCb);
        };
        this.stripe.authenticateSetup(activity, clientSecret);
    };
    Stripe.prototype.confirmPaymentIntent = function (piParams, cb) {
        var _this = this;
        try {
            var activity = application_1.android.foregroundActivity;
            var resultCb_2 = new com.stripe.android.ApiResultCallback({
                onSuccess: function (result) {
                    cb(null, StripePaymentIntent.fromNative(result.getIntent()));
                },
                onError: function (error) {
                    cb(new Error(error.localizedDescription), null);
                }
            });
            activity.onActivityResult = function (requestCode, resultCode, data) {
                _this.stripe.onPaymentResult(requestCode, data, resultCb_2);
            };
            this.stripe.confirmPayment(activity, piParams.native);
        }
        catch (error) {
            cb(new Error(error.localizedDescription), null);
        }
    };
    Stripe.prototype.authenticatePaymentIntent = function (clientSecret, returnUrl, cb) {
        var _this = this;
        var activity = application_1.android.foregroundActivity;
        var resultCb = new com.stripe.android.ApiResultCallback({
            onSuccess: function (result) {
                cb(null, StripePaymentIntent.fromNative(result.getIntent()));
            },
            onError: function (error) {
                cb(new Error(error.localizedDescription), null);
            }
        });
        activity.onActivityResult = function (requestCode, resultCode, data) {
            _this.stripe.onPaymentResult(requestCode, data, resultCb);
        };
        this.stripe.authenticatePayment(activity, clientSecret);
    };
    return Stripe;
}());
exports.Stripe = Stripe;
var Card = (function () {
    function Card(cardNumber, cardExpMonth, cardExpYear, cardCVC) {
        if (cardNumber && cardExpMonth && cardExpYear && cardCVC) {
            this._cardBuilder = new com.stripe.android.model.Card.Builder(cardNumber, new java.lang.Integer(cardExpMonth), new java.lang.Integer(cardExpYear), cardCVC);
        }
    }
    Card.fromNative = function (card) {
        var newCard = new Card(null, null, null, null);
        newCard._cardBuilder = card.toBuilder();
        return newCard;
    };
    Card.fromNativePaymentMethod = function (pm) {
        var pmCard = pm.component8();
        var newCard = new Card(null, null, null, null);
        newCard._last4 = pmCard.component7();
        newCard._brand = toCardBrand(pmCard.component1());
        newCard._cardBuilder = new com.stripe.android.model.Card.Builder(null, pmCard.component4(), pmCard.component5(), null).country(pmCard.component3());
        return newCard;
    };
    Object.defineProperty(Card.prototype, "native", {
        get: function () {
            return this._cardBuilder.build();
        },
        enumerable: true,
        configurable: true
    });
    Card.prototype.validateNumber = function () {
        return this.native.validateNumber();
    };
    Card.prototype.validateCVC = function () {
        return this.native.validateCVC();
    };
    Card.prototype.validateCard = function () {
        return this.native.validateCard();
    };
    Card.prototype.validateExpMonth = function () {
        return this.native.validateExpMonth();
    };
    Card.prototype.validateExpiryDate = function () {
        return this.native.validateExpiryDate();
    };
    Object.defineProperty(Card.prototype, "number", {
        get: function () {
            return this.native.getNumber();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "cvc", {
        get: function () {
            return this.native.getCvc();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "expMonth", {
        get: function () {
            return this.native.getExpMonth().intValue();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "expYear", {
        get: function () {
            return this.native.getExpYear().intValue();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "name", {
        get: function () {
            return this.native.getName();
        },
        set: function (value) {
            this._cardBuilder.name(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressLine1", {
        get: function () {
            return this.native.getAddressLine1();
        },
        set: function (value) {
            this._cardBuilder.addressLine1(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressLine2", {
        get: function () {
            return this.native.getAddressLine2();
        },
        set: function (value) {
            this._cardBuilder.addressLine2(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressCity", {
        get: function () {
            return this.native.getAddressCity();
        },
        set: function (value) {
            this._cardBuilder.addressCity(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressZip", {
        get: function () {
            return this.native.getAddressZip();
        },
        set: function (value) {
            this._cardBuilder.addressZip(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressState", {
        get: function () {
            return this.native.getAddressState();
        },
        set: function (value) {
            this._cardBuilder.addressState(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressCountry", {
        get: function () {
            return this.native.getAddressCountry();
        },
        set: function (value) {
            this._cardBuilder.addressCountry(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "currency", {
        get: function () {
            return this.native.getCurrency();
        },
        set: function (value) {
            this._cardBuilder.currency(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "last4", {
        get: function () {
            if (!this._last4)
                this._last4 = this.native.getLast4();
            return this._last4;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "brand", {
        get: function () {
            if (!this._brand)
                this._brand = toCardBrand(this.native.getBrand());
            return this._brand;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "fingerprint", {
        get: function () {
            return this.native.getFingerprint();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "funding", {
        get: function () {
            return this.native.getFunding();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "country", {
        get: function () {
            return this.native.getCountry();
        },
        enumerable: true,
        configurable: true
    });
    Card.cardImage = function (brand) {
        return getBitmapFromResource(com.stripe.android.model.Card.getBrandIcon(fixupCardBrand(brand)));
    };
    return Card;
}());
exports.Card = Card;
function toCardBrand(brand) {
    if (brand)
        switch (brand.toLowerCase()) {
            case 'visa':
                return 'Visa';
            case 'amex':
            case 'americanexpress':
            case 'american express':
                return 'Amex';
            case 'mastercard':
                return 'MasterCard';
            case 'discover':
                return 'Discover';
            case 'jcb':
                return 'JCB';
            case 'diners':
            case 'dinersclub':
            case 'diners club':
                return 'DinersClub';
            case 'unionpay':
            case 'union pay':
                return 'UnionPay';
        }
    return 'Unknown';
}
function fixupCardBrand(brand) {
    if (brand)
        switch (brand.toLowerCase()) {
            case 'visa':
                return 'Visa';
            case 'amex':
            case 'americanexpress':
            case 'american_express':
            case 'american express':
                return 'American Express';
            case 'mastercard':
                return 'MasterCard';
            case 'discover':
                return 'Discover';
            case 'jcb':
                return 'JCB';
            case 'diners':
            case 'dinersclub':
            case 'diners_club':
            case 'diners club':
                return 'Diners Club';
            case 'unionpay':
            case 'union pay':
                return 'UnionPay';
        }
    return 'Unknown';
}
function getBitmapFromResource(resID) {
    var image = application_1.android.foregroundActivity.getResources().getDrawable(resID, null);
    if (image instanceof android.graphics.Bitmap) {
        return image;
    }
    var bitmap = android.graphics.Bitmap.createBitmap(image.getIntrinsicWidth(), image.getIntrinsicHeight(), android.graphics.Bitmap.Config.ARGB_8888);
    var canvas = new android.graphics.Canvas(bitmap);
    image.setBounds(0, 0, canvas.getWidth(), canvas.getHeight());
    image.draw(canvas);
    return bitmap;
}
var CreditCardView = (function (_super) {
    __extends(CreditCardView, _super);
    function CreditCardView() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(CreditCardView.prototype, "android", {
        get: function () {
            return this._widget;
        },
        enumerable: true,
        configurable: true
    });
    CreditCardView.prototype.createNativeView = function () {
        this._widget = new com.stripe.android.view.CardInputWidget(this._context);
        return this._widget;
    };
    Object.defineProperty(CreditCardView.prototype, "card", {
        get: function () {
            var card = this._widget.getCard();
            if (card) {
                return new Card(card.getNumber(), card.getExpMonth().intValue(), card.getExpYear().intValue(), card.getCvc());
            }
            else {
                return null;
            }
        },
        enumerable: true,
        configurable: true
    });
    return CreditCardView;
}(stripe_common_1.CreditCardViewBase));
exports.CreditCardView = CreditCardView;
var PaymentMethod = (function () {
    function PaymentMethod() {
    }
    PaymentMethod.fromNative = function (native) {
        var pm = new PaymentMethod();
        pm.native = native;
        return pm;
    };
    Object.defineProperty(PaymentMethod.prototype, "id", {
        get: function () { return this.native.component1(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "created", {
        get: function () { return new Date(this.native.component2().longValue() * 1000); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "type", {
        get: function () { return this.native.component4(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "billingDetails", {
        get: function () { return this.native.component5(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "card", {
        get: function () { return Card.fromNativePaymentMethod(this.native); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "customerId", {
        get: function () { return this.native.component6(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "metadata", {
        get: function () { return this.native.component7(); },
        enumerable: true,
        configurable: true
    });
    return PaymentMethod;
}());
exports.PaymentMethod = PaymentMethod;
var StripeIntent = (function () {
    function StripeIntent() {
    }
    Object.defineProperty(StripeIntent.prototype, "id", {
        get: function () { return this.native.getId(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "clientSecret", {
        get: function () { return this.native.getClientSecret(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "description", {
        get: function () { return this.native.getDescription(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "status", {
        get: function () {
            switch (this.native.getStatus()) {
                case com.stripe.android.model.StripeIntent.Status.Canceled:
                    return "canceled";
                case com.stripe.android.model.StripeIntent.Status.Processing:
                    return "processing";
                case com.stripe.android.model.StripeIntent.Status.RequiresAction:
                    return "requires_action";
                case com.stripe.android.model.StripeIntent.Status.RequiresCapture:
                    return "requires_capture";
                case com.stripe.android.model.StripeIntent.Status.RequiresConfirmation:
                    return "requires_confirmation";
                case com.stripe.android.model.StripeIntent.Status.RequiresPaymentMethod:
                    return "requires_payment_method";
                case com.stripe.android.model.StripeIntent.Status.Succeeded:
                    return "succeeded";
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "requiresAction", {
        get: function () {
            return this.status === "requires_action";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "isSuccess", {
        get: function () { return this.status === "succeeded"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "requiresConfirmation", {
        get: function () { return this.status === "requires_confirmation"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "requiresCapture", {
        get: function () { return this.status === "requires_capture"; },
        enumerable: true,
        configurable: true
    });
    return StripeIntent;
}());
var StripePaymentIntent = (function (_super) {
    __extends(StripePaymentIntent, _super);
    function StripePaymentIntent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StripePaymentIntent.fromNative = function (native) {
        var pi = new StripePaymentIntent();
        pi.native = native;
        return pi;
    };
    StripePaymentIntent.fromApi = function (json) {
        var native = com.stripe.android.model.PaymentIntent.fromJson(json);
        return StripePaymentIntent.fromNative(native);
    };
    Object.defineProperty(StripePaymentIntent.prototype, "amount", {
        get: function () { return this.native.getAmount().longValue(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentIntent.prototype, "created", {
        get: function () { return new Date(this.native.getCreated().longValue()); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentIntent.prototype, "currency", {
        get: function () { return this.native.getCurrency(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentIntent.prototype, "captureMethod", {
        get: function () { return this.native.getCaptureMethod(); },
        enumerable: true,
        configurable: true
    });
    return StripePaymentIntent;
}(StripeIntent));
exports.StripePaymentIntent = StripePaymentIntent;
var StripePaymentIntentParams = (function () {
    function StripePaymentIntentParams() {
    }
    Object.defineProperty(StripePaymentIntentParams.prototype, "native", {
        get: function () {
            if (this.sourceId) {
                return com.stripe.android.model.ConfirmPaymentIntentParams.createWithSourceId(this.sourceId, this.clientSecret, this.returnURL);
            }
            else if (this.paymentMethodId) {
                return com.stripe.android.model.ConfirmPaymentIntentParams.createWithPaymentMethodId(this.paymentMethodId, this.clientSecret, this.returnURL);
            }
            else {
                return com.stripe.android.model.ConfirmPaymentIntentParams.create(this.clientSecret);
            }
        },
        enumerable: true,
        configurable: true
    });
    return StripePaymentIntentParams;
}());
exports.StripePaymentIntentParams = StripePaymentIntentParams;
var StripeSetupIntentParams = (function () {
    function StripeSetupIntentParams(paymentMethodId, clientSecret) {
        this.native = com.stripe.android.model.ConfirmSetupIntentParams.create(paymentMethodId, clientSecret);
    }
    return StripeSetupIntentParams;
}());
exports.StripeSetupIntentParams = StripeSetupIntentParams;
var StripeSetupIntent = (function (_super) {
    __extends(StripeSetupIntent, _super);
    function StripeSetupIntent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StripeSetupIntent.fromNative = function (native) {
        var si = new StripeSetupIntent();
        si.native = native;
        return si;
    };
    Object.defineProperty(StripeSetupIntent.prototype, "paymentMethodId", {
        get: function () { return this.native.getPaymentMethodId(); },
        enumerable: true,
        configurable: true
    });
    return StripeSetupIntent;
}(StripeIntent));
exports.StripeSetupIntent = StripeSetupIntent;
