"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var frame_1 = require("tns-core-modules/ui/frame");
var utils_1 = require("tns-core-modules/utils/utils");
var stripe_common_1 = require("./stripe.common");
__export(require("./stripe.common"));
var Stripe = (function () {
    function Stripe(apiKey) {
        STPPaymentConfiguration.sharedConfiguration().publishableKey = apiKey;
    }
    Stripe.prototype.setStripeAccount = function (accountId) {
        STPAPIClient.sharedClient().stripeAccount = accountId;
        STPPaymentConfiguration.sharedConfiguration().stripeAccount = accountId;
    };
    Stripe.prototype.createToken = function (card, cb) {
        if (!card) {
            if (typeof cb === 'function') {
                cb(new Error('Invalid card'), null);
            }
            return;
        }
        var apiClient = STPAPIClient.sharedClient();
        apiClient.createTokenWithCardCompletion(card.native, callback(cb, function (token) { return ({
            id: token.tokenId,
            bankAccount: token.bankAccount,
            card: card,
            created: new Date(token.created),
            livemode: token.livemode,
            android: null,
            ios: token
        }); }));
    };
    Stripe.prototype.createSource = function (card, cb) {
        if (!card) {
            if (typeof cb === 'function') {
                cb(new Error('Invalid card'), null);
            }
            return;
        }
        var sourceParams = STPSourceParams.cardParamsWithCard(card.native);
        var apiClient = STPAPIClient.sharedClient();
        apiClient.createSourceWithParamsCompletion(sourceParams, callback(cb, function (source) { return ({
            id: source.stripeID,
            amount: source.amount,
            card: card,
            clientSecret: source.clientSecret,
            created: new Date(source.created),
            currency: source.currency,
            livemode: source.livemode,
            metadata: source.metadata
        }); }));
    };
    Stripe.prototype.createPaymentMethod = function (card, cb) {
        if (!card) {
            if (typeof cb === 'function') {
                cb(new Error('Invalid card'), null);
            }
            return;
        }
        var apiClient = STPAPIClient.sharedClient();
        var cardParams = STPPaymentMethodCardParams.new();
        if (card.cvc)
            cardParams.cvc = card.cvc;
        if (card.expMonth)
            cardParams.expMonth = card.expMonth;
        if (card.expYear)
            cardParams.expYear = card.expYear;
        if (card.number)
            cardParams.number = card.number;
        var billing = STPPaymentMethodBillingDetails.new();
        billing.address = STPPaymentMethodAddress.new();
        if (card.addressLine1)
            billing.address.line1 = card.addressLine1;
        if (card.addressLine2)
            billing.address.line2 = card.addressLine2;
        if (card.addressCity)
            billing.address.city = card.addressCity;
        if (card.addressState)
            billing.address.state = card.addressState;
        if (card.addressZip)
            billing.address.postalCode = card.addressZip;
        if (card.addressCountry)
            billing.address.country = card.addressCountry;
        var params = STPPaymentMethodParams.paramsWithCardBillingDetailsMetadata(cardParams, billing, null);
        apiClient.createPaymentMethodWithParamsCompletion(params, callback(cb, function (pm) { return PaymentMethod.fromNative(pm); }));
    };
    Stripe.prototype.retrievePaymentIntent = function (clientSecret, cb) {
        var apiClient = STPAPIClient.sharedClient();
        apiClient.retrievePaymentIntentWithClientSecretCompletion(clientSecret, callback(cb, function (pi) { return StripePaymentIntent.fromNative(pi); }));
    };
    Stripe.prototype.confirmSetupIntent = function (paymentMethodId, clientSecret, cb) {
        STPPaymentHandler.sharedHandler().confirmSetupIntentWithAuthenticationContextCompletion(new StripeSetupIntentParams(paymentMethodId, clientSecret).native, this._getAuthentificationContext(), function (status, si, error) {
            if (error) {
                cb(new Error(error.toLocaleString()), null);
            }
            else {
                cb(null, StripeSetupIntent.fromNative(si));
            }
        });
    };
    Stripe.prototype.authenticateSetupIntent = function (clientSecret, returnUrl, cb) {
        STPPaymentHandler.sharedHandler().handleNextActionForSetupIntentWithAuthenticationContextReturnURLCompletion(clientSecret, this._getAuthentificationContext(), returnUrl, function (status, pi, error) {
            if (error) {
                cb(new Error(error.toLocaleString()), null);
            }
            else {
                cb(null, StripeSetupIntent.fromNative(pi));
            }
        });
    };
    Stripe.prototype.confirmPaymentIntent = function (params, cb) {
        STPPaymentHandler.sharedHandler().confirmPaymentWithAuthenticationContextCompletion(params.native, this._getAuthentificationContext(), function (status, pi, error) {
            if (error) {
                cb(new Error(error.toLocaleString()), null);
            }
            else {
                cb(null, StripePaymentIntent.fromNative(pi));
            }
        });
    };
    Stripe.prototype.authenticatePaymentIntent = function (clientSecret, returnUrl, cb) {
        STPPaymentHandler.sharedHandler().handleNextActionForPaymentWithAuthenticationContextReturnURLCompletion(clientSecret, this._getAuthentificationContext(), returnUrl, function (status, pi, error) {
            if (error) {
                cb(new Error(error.toLocaleString()), null);
            }
            else {
                cb(null, StripePaymentIntent.fromNative(pi));
            }
        });
    };
    Stripe.prototype._getAuthentificationContext = function () {
        var authContext = STPPaymentContext.alloc();
        var rootVC = frame_1.topmost().currentPage.ios;
        authContext.hostViewController = utils_1.ios.getVisibleViewController(rootVC);
        authContext.authenticationPresentingViewController = function () {
            return authContext.hostViewController;
        };
        return authContext;
    };
    return Stripe;
}());
exports.Stripe = Stripe;
function callback(cb, cvt) {
    return function (value, error) {
        if (!error) {
            if (typeof cb === 'function') {
                cb(null, cvt(value));
            }
        }
        else {
            if (typeof cb === 'function') {
                cb(new Error(error.toLocaleString()), null);
            }
        }
    };
}
var Card = (function () {
    function Card(cardNumber, cardExpMonth, cardExpYear, cardCVC) {
        if (cardNumber && cardExpMonth && cardExpYear && cardCVC) {
            this.native = STPCardParams.alloc().init();
            this.native.number = cardNumber;
            this.native.expMonth = cardExpMonth;
            this.native.expYear = cardExpYear;
            this.native.cvc = cardCVC;
        }
    }
    Card.fromNative = function (card) {
        var newCard = new Card(null, null, null, null);
        newCard.native = card;
        return newCard;
    };
    Card.fromNativePaymentMethod = function (pm) {
        var newCard = new Card(null, null, null, null);
        var card = STPCardParams.alloc().init();
        card.addressCountry = pm.card.country;
        card.expMonth = pm.card.expMonth;
        card.expYear = pm.card.expYear;
        newCard._last4 = pm.card.last4;
        newCard._brand = Card.toCardBrand(pm.card.brand);
        newCard.native = card;
        return newCard;
    };
    Card.prototype.validateNumber = function () {
        var isValid = false;
        var state = STPCardValidator.validationStateForNumberValidatingCardBrand(this.native.number, true);
        switch (state) {
            case 0:
                isValid = true;
                break;
            case 2:
                isValid = false;
                break;
            case 1:
                isValid = false;
                break;
        }
        return isValid;
    };
    Card.prototype.validateCVC = function () {
        var isValid = false;
        var brand = STPCardValidator.brandForNumber(this.native.number);
        var state = STPCardValidator.validationStateForCVCCardBrand(this.native.cvc, brand);
        switch (state) {
            case 0:
                isValid = true;
                break;
            case 2:
                isValid = false;
                break;
            case 1:
                isValid = false;
                break;
        }
        return isValid;
    };
    Card.prototype.validateCard = function () {
        try {
            return (STPCardValidator.validationStateForCard(this.native) ===
                0);
        }
        catch (ex) {
            return false;
        }
    };
    Card.prototype.validateExpMonth = function () {
        var isValid = false;
        var state = STPCardValidator.validationStateForExpirationMonth(String(this.native.expMonth));
        switch (state) {
            case 0:
                isValid = true;
                break;
            case 2:
                isValid = false;
                break;
            case 1:
                isValid = false;
                break;
        }
        return isValid;
    };
    Card.prototype.validateExpiryDate = function () {
        var isValid = false;
        var state = STPCardValidator.validationStateForExpirationYearInMonth(String(this.native.expYear), String(this.native.expMonth));
        switch (state) {
            case 0:
                isValid = true;
                break;
            case 2:
                isValid = false;
                break;
            case 1:
                isValid = false;
                break;
        }
        return isValid;
    };
    Object.defineProperty(Card.prototype, "number", {
        get: function () {
            return this.native.number;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "cvc", {
        get: function () {
            return this.native.cvc;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "expMonth", {
        get: function () {
            return this.native.expMonth;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "expYear", {
        get: function () {
            return this.native.expYear;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "name", {
        get: function () {
            return this.native.name;
        },
        set: function (value) {
            this.native.name = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressLine1", {
        get: function () {
            return this.native.addressLine1;
        },
        set: function (value) {
            this.native.addressLine1 = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressLine2", {
        get: function () {
            return this.native.addressLine2;
        },
        set: function (value) {
            this.native.addressLine2 = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressCity", {
        get: function () {
            return this.native.addressCity;
        },
        set: function (value) {
            this.native.addressCity = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressZip", {
        get: function () {
            return this.native.addressZip;
        },
        set: function (value) {
            this.native.addressZip = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressState", {
        get: function () {
            return this.native.addressState;
        },
        set: function (value) {
            this.native.addressState = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "addressCountry", {
        get: function () {
            return this.native.addressCountry;
        },
        set: function (value) {
            this.native.addressCountry = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "currency", {
        get: function () {
            return this.native.currency;
        },
        set: function (value) {
            this.native.currency = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "last4", {
        get: function () {
            if (!this._last4)
                this._last4 = this.native.last4();
            return this._last4;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "brand", {
        get: function () {
            if (!this._brand)
                this._brand = Card.toCardBrand(STPCardValidator.brandForNumber(this.native.number));
            return this._brand;
        },
        enumerable: true,
        configurable: true
    });
    Card.toCardBrand = function (brand) {
        switch (brand) {
            case 0:
                return 'Visa';
            case 1:
                return 'Amex';
            case 2:
                return 'MasterCard';
            case 3:
                return 'Discover';
            case 4:
                return 'JCB';
            case 5:
                return 'DinersClub';
            case 6:
                return 'UnionPay';
        }
        return 'Unknown';
    };
    Card.fromCardBrand = function (brand) {
        switch (brand.toLowerCase()) {
            case 'visa':
                return 0;
            case 'amex':
            case 'americanexpress':
            case 'american_express':
            case 'american express':
                return 1;
            case 'mastercard':
                return 2;
            case 'discover':
                return 3;
            case 'jcb':
                return 4;
            case 'dinersclub':
            case 'diners_club':
            case 'diners club':
                return 5;
            case 'unionpay':
            case 'union pay':
                return 6;
        }
        return 7;
    };
    Object.defineProperty(Card.prototype, "fingerprint", {
        get: function () {
            return '';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "funding", {
        get: function () {
            return '';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Card.prototype, "country", {
        get: function () {
            return this.native.addressCountry;
        },
        enumerable: true,
        configurable: true
    });
    Card.cardImage = function (brand) {
        return STPImageLibrary.brandImageForCardBrand(Card.fromCardBrand(brand));
    };
    return Card;
}());
exports.Card = Card;
var CreditCardView = (function (_super) {
    __extends(CreditCardView, _super);
    function CreditCardView() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CreditCardView.prototype.createNativeView = function () {
        return STPPaymentCardTextField.alloc().initWithFrame(CGRectMake(10, 10, 300, 44));
    };
    CreditCardView.prototype.initNativeView = function () {
        this.nativeView.owner = this;
        _super.prototype.initNativeView.call(this);
    };
    CreditCardView.prototype.disposeNativeView = function () {
        this.nativeView.owner = null;
        _super.prototype.disposeNativeView.call(this);
    };
    Object.defineProperty(CreditCardView.prototype, "card", {
        get: function () {
            try {
                var stpCardParams = STPCardParams.alloc();
                stpCardParams.cvc = this.nativeView.cardParams.cvc;
                stpCardParams.number = this.nativeView.cardParams.number;
                stpCardParams.expMonth = this.nativeView.cardParams.expMonth;
                stpCardParams.expYear = this.nativeView.cardParams.expYear;
                var valid = STPCardValidator.validationStateForCard(stpCardParams) ===
                    0;
                return valid
                    ? new Card(this.nativeView.cardParams.number, this.nativeView.cardParams.expMonth, this.nativeView.cardParams.expYear, this.nativeView.cardParams.cvc)
                    : null;
            }
            catch (ex) {
                return null;
            }
        },
        enumerable: true,
        configurable: true
    });
    return CreditCardView;
}(stripe_common_1.CreditCardViewBase));
exports.CreditCardView = CreditCardView;
var PaymentMethod = (function () {
    function PaymentMethod() {
    }
    PaymentMethod.fromNative = function (native) {
        var pm = new PaymentMethod();
        pm.native = native;
        return pm;
    };
    Object.defineProperty(PaymentMethod.prototype, "id", {
        get: function () { return this.native.stripeId; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "created", {
        get: function () { return new Date(this.native.created); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "type", {
        get: function () { return this.native.type === 0 ? "card" : null; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "billingDetails", {
        get: function () { return this.native.billingDetails; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "card", {
        get: function () { return Card.fromNativePaymentMethod(this.native); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "customerId", {
        get: function () { return this.native.customerId; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PaymentMethod.prototype, "metadata", {
        get: function () { return this.native.metadata; },
        enumerable: true,
        configurable: true
    });
    return PaymentMethod;
}());
exports.PaymentMethod = PaymentMethod;
var StripeIntent = (function () {
    function StripeIntent() {
    }
    Object.defineProperty(StripeIntent.prototype, "created", {
        get: function () { return new Date(this.native.created); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "clientSecret", {
        get: function () { return this.native.clientSecret; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "status", {
        get: function () {
            switch (this.native.status) {
                case 7:
                    return "canceled";
                case 4:
                    return "processing";
                case 3:
                    return "requires_action";
                case 6:
                    return "requires_capture";
                case 2:
                    return "requires_confirmation";
                case 1:
                    return "requires_payment_method";
                case 5:
                    return "succeeded";
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "requiresAction", {
        get: function () { return this.native.status === 3; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "isSuccess", {
        get: function () { return this.status === "succeeded"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "requiresConfirmation", {
        get: function () { return this.status === "requires_confirmation"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "requiresCapture", {
        get: function () { return this.status === "requires_capture"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeIntent.prototype, "description", {
        get: function () { return this.native.description; },
        enumerable: true,
        configurable: true
    });
    return StripeIntent;
}());
var StripePaymentIntent = (function (_super) {
    __extends(StripePaymentIntent, _super);
    function StripePaymentIntent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StripePaymentIntent.fromNative = function (native) {
        var pi = new StripePaymentIntent();
        pi.native = native;
        return pi;
    };
    StripePaymentIntent.fromApi = function (json) {
        var native = STPPaymentIntent.decodedObjectFromAPIResponse(json);
        return StripePaymentIntent.fromNative(native);
    };
    Object.defineProperty(StripePaymentIntent.prototype, "id", {
        get: function () { return this.native.stripeId; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentIntent.prototype, "amount", {
        get: function () { return this.native.amount; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentIntent.prototype, "currency", {
        get: function () { return this.native.currency; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripePaymentIntent.prototype, "captureMethod", {
        get: function () {
            switch (this.native.captureMethod) {
                case 1:
                    return "automatic";
                case 2:
                    return "manual";
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    return StripePaymentIntent;
}(StripeIntent));
exports.StripePaymentIntent = StripePaymentIntent;
var StripePaymentIntentParams = (function () {
    function StripePaymentIntentParams() {
    }
    Object.defineProperty(StripePaymentIntentParams.prototype, "native", {
        get: function () {
            var n = STPPaymentIntentParams.alloc().initWithClientSecret(this.clientSecret);
            n.paymentMethodParams = this.paymentMethodParams;
            n.paymentMethodId = this.paymentMethodId;
            n.sourceParams = this.sourceParams;
            n.sourceId = this.sourceId;
            n.returnURL = this.returnURL;
            return n;
        },
        enumerable: true,
        configurable: true
    });
    return StripePaymentIntentParams;
}());
exports.StripePaymentIntentParams = StripePaymentIntentParams;
var StripeSetupIntent = (function (_super) {
    __extends(StripeSetupIntent, _super);
    function StripeSetupIntent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StripeSetupIntent.fromNative = function (native) {
        var si = new StripeSetupIntent();
        si.native = native;
        return si;
    };
    Object.defineProperty(StripeSetupIntent.prototype, "id", {
        get: function () { return this.native.stripeID; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StripeSetupIntent.prototype, "paymentMethodId", {
        get: function () { return this.native.paymentMethodID; },
        enumerable: true,
        configurable: true
    });
    return StripeSetupIntent;
}(StripeIntent));
exports.StripeSetupIntent = StripeSetupIntent;
var StripeSetupIntentParams = (function () {
    function StripeSetupIntentParams(paymentMethodId, clientSecret) {
        this.native = STPSetupIntentConfirmParams.alloc();
        this.native.paymentMethodID = paymentMethodId;
        this.native.clientSecret = clientSecret;
    }
    return StripeSetupIntentParams;
}());
exports.StripeSetupIntentParams = StripeSetupIntentParams;
var StripeRedirectSession = (function () {
    function StripeRedirectSession(paymentIntent, cb) {
        this.native = STPRedirectContext.alloc().initWithPaymentIntentCompletion(paymentIntent.native, callback(cb, function (clientSecret) { return clientSecret; }));
    }
    StripeRedirectSession.prototype.startRedirectFlow = function (view) {
        this.native.startRedirectFlowFromViewController(view.viewController);
    };
    StripeRedirectSession.prototype.cancel = function () {
        this.native.cancel();
    };
    return StripeRedirectSession;
}());
exports.StripeRedirectSession = StripeRedirectSession;
var StripeRedirectState;
(function (StripeRedirectState) {
    StripeRedirectState[StripeRedirectState["NotStarted"] = 0] = "NotStarted";
    StripeRedirectState[StripeRedirectState["InProgress"] = 1] = "InProgress";
    StripeRedirectState[StripeRedirectState["Cancelled"] = 2] = "Cancelled";
    StripeRedirectState[StripeRedirectState["Completed"] = 3] = "Completed";
})(StripeRedirectState = exports.StripeRedirectState || (exports.StripeRedirectState = {}));
